(*****************************************************************************
 *          HOL-TestGen - specification based test case generation
 *                                                                            
 * server.sml - Testing a SML List Implementation
 * Copyright (C) 2005 Achim D. Brucker <brucker@inf.ethz.ch>   
 *                    Burkhart Wolff   <bwolff@inf.ethz.ch>    
 *                                                                            
 * This file is part of HOL-TestGen.                  
 *                                                                            
 * HOL-TestGen is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free   
 * Software Foundation; either version 2 of the License, or (at your option)  
 * any later version.                                                         
 *                                                                            
 * HOL-TestGen is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
 * more details.                                                              
 *                                                                            
 * You should have received a copy of the GNU General Public License along    
 * with this program; if not, write to the Free Software Foundation, Inc.,    
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                  
 ******************************************************************************)

structure server = struct

datatype vars = X | Y;
datatype data = Data;

datatype InEvent = req of int 
		 | reqA of vars
		 | send of data * int 
		 | sendA of data * vars 
		 | stop;

datatype OutEvent = port of int 
		  | portA of vars 
		  | ack;

fun vars2String X = "X"
  | vars2String Y = "Y"

fun InEvent2String (req x)   = "req("^(Int.toString x)^")"
  | InEvent2String (reqA x)  = "reqA("^(vars2String x)^")"
  | InEvent2String (send (x,y))  = "send(Data, port:"^(Int.toString y)^")"
  | InEvent2String (sendA x) = "sendA "
  | InEvent2String stop      = "stop  ";

fun OutEvent2String (port x)   = "port("^(Int.toString x)^")"
  | OutEvent2String (portA x)  = "portA("^(vars2String x)^")"
  | OutEvent2String  ack       = "ack"


fun implementation (req x)   = if (x-1) >=0 then port (x-1) else port 0
  | implementation (reqA x)  = ack
  | implementation (send x)  = ack
  | implementation (sendA x) = ack
  | implementation stop      = ack

fun implementation' (req x)   = 5
  | implementation' (reqA x)  = ack
  | implementation' (send x)  = ack
  | implementation' (sendA x) = ack
  | implementation' stop      = ack

fun read event  =  
    let
	val _ = print ("received: "^(InEvent2String event))
        val ans = implementation event 
	val _ = print ("\t -> sending: "^(OutEvent2String ans)^("\n"))
    in
	ans
    end 


end

